/*===========================================================================*/
/*
 * This file is part of
 * libalinga - a C++ library for Alligned Linguistic Annotation
 *
 * Copyright (C) 2007, 2008 Elaine Tsiang YueLien
 *
 * libalinga is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, see http://www.gnu.org/licenses, or write to the
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301, USA
 *
 *===========================================================================*/
/** @file Encode.H                                                           *
 *                                                                           *
 *  ALingA::Encode interface                                                 *
 *                                                                           */
/*===========================================================================*/
#ifndef	EncodeInterface
#define	EncodeInterface

#ifdef __GNUG__
#pragma interface
#endif

#include <glibmm/ustring.h>

namespace ALingA
{
  /// @brief Supported data encoding types
  enum Encoding{
    Unspecified
    ,Signed16Bit
    ,Unsigned16Bit
    ,Signed8Bit
    ,Unsigned8Bit
    ,Float32Bit
  };

  /// @brief For encoding to and decoding from network bigendian.
  class	Encode
  {
  private:
    void *	impl_;

  public:
    /// @brief Test for host endianness.
    static
    bool
    hostIsBigEndian();

    /// @brief Unspecified Encoding defaults to unsigned 16bit
    Encode();
      
    Encode(Encoding);

    /// @brief Using the corresponding Encoding string, e.g. "Signed16Bit"
    Encode(Glib::ustring);

    /// @brief Copy construction
    Encode(Encode &);

    ~Encode();

    Encode &
    operator =(Encode &);

    Encoding
    encoding();

    bool
    floating();

    bool
    isSigned();

    size_t
    dataSize();

    void
    encode(
	   void *	data
	   ,size_t	size
	   );

    void
    decode(
	   void*	data
	   ,size_t	size
	   );

    Glib::ustring
    inText();

  };
}
#endif
