/*===========================================================================*/
/*
 * This file is part of
 * libalinga - a C++ library for Alligned Linguistic Annotation
 *
 * Copyright (C) 2007, 2008 Elaine Tsiang YueLien
 *
 * libalinga is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, see http://www.gnu.org/licenses, or write to the
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301, USA
 *
 *===========================================================================*/
/** @file Metas.H                                                         *
 *                                                                           *
 *  ALingA::Metas interface                                               *
 *                                                                           */
/*===========================================================================*/
#ifndef	MetasInterface
#define	MetasInterface

#ifdef __GNUG__
#pragma interface
#endif

#include <ALingA/XMLparser.H>

#include <glibmm/ustring.h>
#include <map>

namespace ALingA
{
  using Glib::ustring;
  using std::map;

  typedef Glib::ustring				MetaName;
  typedef Glib::ustring				MetaValue;
  typedef map<MetaName, MetaValue>		Metas;
  typedef Metas::iterator			iMetas;


  class MetasXMLparser : public XMLparser
  {
  private:
    void *		impl_;

  public:
    MetasXMLparser(
		   XMLparser &	parent
		   ,Metas &
		   );

    ~MetasXMLparser();

    void
    on_start_document()
    {
      throw XMLparser::UnexpectedDocument();
    }
    

    void
    on_end_document()
    {
      throw XMLparser::UnexpectedDocument();
    }
    


    void
    on_start_element(
		     const Glib::ustring& 			name
		     , const xmlpp::SaxParser::AttributeList&	attributes
		     );
    void
    on_end_element(
		   const Glib::ustring& 	name
		   );
    
    void
    on_characters(
		  const Glib::ustring& characters
		  );

    void on_cdata_block(
			const Glib::ustring& text
			)
    {
      throw XMLparser::UnexpectedCData(text);
    }
    
  };
}
#endif
