/*===========================================================================*/
/*
 * This file is part of
 * libalinga - a C++ library for Alligned Linguistic Annotation
 *
 * Copyright (C) 2007, 2008 Elaine Tsiang YueLien
 *
 * libalinga is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, see http://www.gnu.org/licenses, or write to the
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301, USA
 *
 *===========================================================================*/
/** @file EncodeImpl.C                                                       *
 *                                                                           *
 *  ALingA::EncodeImpl - Encode implementation                               *
 *                                                                           */
/*===========================================================================*/
#ifdef __GNUG__
#pragma implementation "Encode.H"
#pragma implementation "EncodeImpl.H"
#endif

#include	<EncodeImpl.H>

namespace	ALingA
{
  //--Encode--------------------------------------------------------------------------------

  map<Glib::ustring, Encoding>		EncodeImpl::encodings_;
  map<Encoding, Glib::ustring>		EncodeImpl::decodings_;
  bool					EncodeImpl::hostIsBigEndian_;

  void
  EncodeImpl::initCodings()
  {
    if ( encodings_.empty() )
      {
	encodings_["Signed16Bit"] = Signed16Bit;
	encodings_["Unsigned16Bit"] = Unsigned16Bit;
	encodings_["Signed8Bit"] = Signed8Bit;
	encodings_["Unsigned8Bit"] = Unsigned8Bit;
	encodings_["Float32Bit"] = Float32Bit;

	decodings_[Signed16Bit] = "Signed16Bit";
	decodings_[Unsigned16Bit] = "Unsigned16Bit";
	decodings_[Signed8Bit] = "Signed8Bit";
	decodings_[Unsigned8Bit] = "Unsigned8Bit";
	decodings_[Float32Bit] = "Float32Bit";
      }

    long test = 1;
    hostIsBigEndian_ = *(reinterpret_cast<char *>(&test))? false : true;

    Encode::hostIsBigEndian();
  }

  bool
  Encode::hostIsBigEndian()
  {
    long test = 1;
    EncodeImpl::hostIsBigEndian_ = *(reinterpret_cast<char *>(&test))? false : true;

    return(EncodeImpl::hostIsBigEndian_);
  }

  Encode::Encode(Encoding encoding)
  {
    impl_ = new EncodeImpl(encoding);
  }

  EncodeImpl::EncodeImpl(Encoding encoding)
    : encoding_(encoding)
  {
    initCodings();

    if ( (encoding == Signed16Bit)
	 ||
	 (encoding == Signed8Bit)
	 ||
	 (encoding == Float32Bit)
	 )
      isSigned_ = true;
    else
      isSigned_ = false;
      
    if ( encoding == Float32Bit )
      floating_ = true;
    else
      floating_ = false;

    if ( (encoding == Signed16Bit)
	 ||
	 (encoding == Unsigned16Bit)
	 )
      dataSize_ = 2;
    else if (
	     (encoding == Signed8Bit)
	     ||
	     (encoding == Unsigned8Bit)
	     )
      dataSize_ = 1;
    else if ( (encoding == Float32Bit)
	      )
      dataSize_ = 4;
    else
      dataSize_ = 2;
  }

  Encode::Encode()
  {
    new(this) Encode(Unsigned16Bit);
  }
  
  Encode::Encode(Glib::ustring	text)
  {
    new(this) Encode(EncodeImpl::encodings_[text]);
  }

  Encode::Encode(Encode & enc)
  {
    new(this) Encode(enc.encoding());
  }

  Encode &
  Encode::operator=(Encode & enc)
  {
    EncodeImpl * impl = static_cast<EncodeImpl *>(impl_);
    EncodeImpl * impl1 = static_cast<EncodeImpl *>(enc.impl_);

    impl->encoding_ = impl1->encoding_;
    impl->isSigned_ = impl1->isSigned_;
    impl->floating_ = impl1->floating_;
    impl->dataSize_ = impl1->dataSize_;

    return(*this);
  }

  Encode::~Encode()
  {
    if ( impl_ )
      {
	EncodeImpl * impl = static_cast<EncodeImpl *>(impl_);
	delete impl;
      }
  }

  Encoding
  Encode::encoding()
  {
    EncodeImpl * impl = static_cast<EncodeImpl *>(impl_);

    return(impl->encoding_);
  }

  bool
  Encode::floating()
  {
    EncodeImpl * impl = static_cast<EncodeImpl *>(impl_);

    return(impl->floating_);
  }
    
  bool
  Encode::isSigned()
  {
    EncodeImpl * impl = static_cast<EncodeImpl *>(impl_);

    return(impl->isSigned_);
  }
    
  size_t
  Encode::dataSize()
  {
    EncodeImpl * impl = static_cast<EncodeImpl *>(impl_);

    return(impl->dataSize_);
  }

  void
  EncodeImpl::reverse(
		      void *	data
		      ,size_t	size
		      )
  {
    unsigned char * cs = static_cast<unsigned char *>(data);

    if ( !hostIsBigEndian_ )
      {
	if ( size == 2 )
	  {
	    unsigned char c = cs[0];
	    cs[0] = cs[1];
	    cs[1] = c;
	  }
	else if ( size == 4 )
	  {
	    unsigned char c = cs[0];
	    cs[0] = cs[3];
	    cs[3] = c;
	    c = cs[1];
	    cs[1] = cs[2];
	    cs[2] = c;
	  }
	else
	  {
	    for ( ;
		  size > 0
		    ;
		  size -= dataSize_
		    ,cs += dataSize_
		    
		  )
	      {
		reverse(cs
			,dataSize_
			);
	      }
	  }
      }
  }

  void
  Encode::encode(
		 void *		data
		 ,size_t	size
		 )
  {
    EncodeImpl * impl = static_cast<EncodeImpl *>(impl_);

    impl->reverse(data
		  ,size
		  );
  }

  void
  Encode::decode(
		 void *		data
		 ,size_t	size
	       )
  {
    EncodeImpl * impl = static_cast<EncodeImpl *>(impl_);
    
    impl->reverse(data
		  ,size
		  );
  }

  Glib::ustring
  Encode::inText()
  {
    EncodeImpl * impl = static_cast<EncodeImpl *>(impl_);
    
    return(EncodeImpl::decodings_[impl->encoding_]);
  }
}
