/*===========================================================================*/
/*
 * This file is part of
 * libalinga - a C++ library for Alligned Linguistic Annotation
 *
 * Copyright (C) 2007, 2008 Elaine Tsiang YueLien
 *
 * libalinga is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, see http://www.gnu.org/licenses, or write to the
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301, USA
 *
 *===========================================================================*/
/** @file NodesArcsImpl.H                                                    *
 *                                                                           *
 *  ALingA::NodeImpl                                                         *
 *  ALingA::ArcImpl                                                          *
 *  ALingA::Nodes                                                            *
 *  ALingA::Arcs                                                             *
 *                                                                           */
/*===========================================================================*/
#ifndef	NodesArcsImplementationInterface
#define	NodesArcsImplementationInterface

#ifdef __GNUG__
#pragma interface
#endif

#include <ALingA/XMLwriter.H>

#include <set>

#include	<ALingA/NodesArcs.H>

namespace ALingA
{
  typedef TypedAttribute<Id>	IdAttribute;

  class				NodeImpl;
  class				ArcImpl;
}

#include	<LingAImpl.H>

namespace	ALingA
{
  class BoxRelation
  {
  public:
    enum Type
      {
	BELOW = -3
	,LOWER = -2
	,CONTAINED = -1
	,EQUAL = 0
	,CONTAINING = 1
	,HIGHER = 2
	,ABOVE = 3
      };

    Type
    operator()(const Box &e1, const Box &e2) const
    {
      if ( e1.right < e2.left )
	return BELOW;

      if ( e2.right < e1.left )
	return ABOVE;

      // e1.right >= e2.left and e2.right >= e1.left

      if ( e1.left < e2.left )
	{
	  if ( e1.right <= e2.right )
	    return LOWER;
	  else // e1.right > e2.right
	    return CONTAINING;
	}
	else if ( e1.left == e2.left )
	  {
	    if ( e1.right < e2.right )
	      return LOWER;
	    else if ( e1.right == e2.right )
	      return EQUAL;
	    else // e1.right > e2.right
	      return HIGHER;
	  }
	else // e1.left > e2.left
	  {
	    if ( e1.right < e2.right )
	      return CONTAINED;
	    else // ( e1.right >= e2.right )
	      return HIGHER;
	  }
      
    }
  };

  inline
  bool operator<(const Box & b1, const Box & b2)
  {
    // may need a Box::rectify
    // {
    Box box1(b1.left, b1.right);
    Box box2(b2.left, b2.right);

    if ( b1.left > b1.right )
      {
	box1.left = b1.right;
	box1.right = b1.left;
      }
    if ( b2.left > b2.right )
      {
	box2.left = b2.right;
	box2.right = b2.left;
      }
    // }

    BoxRelation r;
    if ( int(r(box1,box2)) < 0 )
      return true;
    else
      return false;
  }

  class ArcImpl
  {
    friend class Node;
    friend class iNode;
    friend class Arc;
    friend class iArc;
    friend class LingA;
    friend class LingAImpl;
    friend class NodeXMLparser;
    friend class ArcXMLparser;
    friend class LabelXMLparser;
    friend class PrecedingArc;

    friend bool operator < (const Arc &, const Arc &);
    
  private:
    Arc *			intf_;

    Id				id_;
    NodeImpl *			left_;
    NodeImpl *			right_;

    Label			label_;

    Metas *			metas_;

    LingAImpl &			linga_;

    bool			junk_;

    ArcImpl(
	    Id
	    ,NodeImpl &		left
	    ,NodeImpl &		right
	    ,Label
	    ,Arc *
	    );
    ~ArcImpl();

    bool
    epochDefined();

    // Detach this ArcImpl from its left NodeImpl
    void
    detachLeft();

    // Detach this ArcImpl from its right NodeImpl
    void
    detachRight();
  };

  class NodeImpl
  {
    friend class Node;
    friend class iNode;
    friend class Arc;
    friend class ArcImpl;
    friend class iArc;
    friend class LingA;
    friend class LingAImpl;
    friend class NodeXMLparser;
    friend class ArcXMLparser;
    friend class PrecedingNode;

    friend bool operator < (const Node &, const Node &);

  private:
    Node *			intf_;

    Id				id_;
    Arcs			lefts_;
    Arcs			rights_;
    long long			time_;
    long long			space_;

    LingAImpl &			linga_;

    bool			written_;
    bool			junk_;

    NodeImpl(
	     LingAImpl &
	     ,Id
	     ,long long	time
	     ,long long space
	     );

    ~NodeImpl();

    void
    time(Position);

    bool
    detached();
  };
}

#endif



