/*===========================================================================*/
/*
 * This file is part of
 * libalinga - a C++ library for Alligned Linguistic Annotation
 *
 * Copyright (C) 2007, 2008 Elaine Tsiang YueLien
 *
 * libalinga is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, see http://www.gnu.org/licenses, or write to the
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301, USA
 *
 *===========================================================================*/
/** @file SignalImpl.H                                                       *
 *                                                                           *
 *  ALingA::Signal implementation interface                                  *
 *                                                                           */
/*===========================================================================*/
#ifndef	SignalImplementationInterface
#define	SignalImplementationInterface

#ifdef __GNUG__
#pragma interface
#endif

#include <ALingA/Signal.H>

namespace ALingA
{
  class SignalImpl;
}

#include <ALingAImpl.H>

namespace ALingA
{
  class SignalImpl : public Signal
  {
    friend class	Signal;
    friend class	ALingAImpl;
    friend class	DriverXMLparser;
    friend class 	ALingAXMLparser;
    friend class 	SignalXMLparser;


  private:
    long		track_;
    size_t		baseGransPerFrame_;
    Coordinate		bias_;
    Decimation		decimation_;


    /// these fields are user-modifiable
    pthread_cond_t	condType_;
    pthread_mutex_t	lockType_;
    Glib::ustring	codecType_;
    Glib::ustring	href_;
    Metas		metas_;
    
    SignalImpl(
	       long		trk
	       ,size_t		bgpf = 1
	       ,Coordinate	b = Coordinate(0,0)
	       ,Decimation	d = Decimation(1,1)
	       )
      : track_(trk)
      , baseGransPerFrame_(bgpf)
      , bias_(b)
      , decimation_(d)
      , codecType_()
      , href_()
      , metas_()
    {
      pthread_mutex_init(&lockType_, 0);
      pthread_cond_init(&condType_, 0);
    }

    ~SignalImpl()
    {}

    void
    codecTypeFilled();

    void
    codecTypeEmpty();

    void
    codecType(
	      Glib::ustring
	      );

    Coordinate
    baseCoordinate(Coordinate signalCoordinate) const;

    Coordinate
    signalCoordinate(Coordinate baseCoordinate) const;
    
    Position
    basePosition(Coordinate	signalCoordinate) const;

  };
}
#endif
