/*===========================================================================*/
/*
 * This file is part of
 * libalinga - a C++ library for Alligned Linguistic Annotation
 *
 * Copyright (C) 2007, 2008 Elaine Tsiang YueLien
 *
 * libalinga is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, see http://www.gnu.org/licenses, or write to the
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301, USA
 *
 *===========================================================================*/
/** @file XMLparserImpl.H                                                     *
 *                                                                           *
 *  ALingA::XMLparser implementation interface                                *
 *                                                                           */
/*===========================================================================*/
#ifndef	XMLparserImplementationInterface
#define	XMLparserImplementationInterface

#ifdef __GNUG__
#pragma interface
#endif

#include <ALingA/XMLparser.H>
#include <libxml++/libxml++.h>
#include <stack>

namespace	ALingA
{
  using xmlpp::SaxParser;

  class XMLparserImpl : public SaxParser
  {
    friend class	XMLparser;

  private:
    XMLparser *				intf_;

    Glib::ustring			elementName_;
    std::stack<XMLparserImpl *> *	context_;
    

    // if no parent, makes stack
    // otherwise makes parent's stack its own stack
    // pushes itself onto the stack
    XMLparserImpl(
		  Glib::ustring	elementName
		  ,XMLparserImpl * 	= 0	// parent
		  );

    // if no parent, then ends parse_chunk
    // pops itself off its stack
    // destroys stack if empty
    ~XMLparserImpl();

    // these on_* callbacks call the corresponding XMLparser callbacks
    // which defaults to the base SaxParser callbacks if not overridden
    void on_start_document()
    {
      intf_->on_start_document();
    }

    void on_end_document()
    {
      intf_->on_end_document();
    }

    // start_element is handling a contained element, which may vary
    // it tests the arg name for which element is this current invocation,
    // derive: gets or creates the pointer to the receiving data block,
    // and creates the parser with that pointer and the attributes
    void
    on_start_element(
		     const Glib::ustring& name
		     , const xmlpp::SaxParser::AttributeList& attributes
		     )
    {
      // dispatcher to the different levels of XMLparsers
      if ( !context_->empty() )
	context_->top()->intf_->on_start_element(
						 name
						 ,attributes
						 );
    }

    // looks at the stack top parser, and verifies that corresponds
    // to the passed name, otherwise throw BadEndElement
    // deletes the stack top parser (which causes it to be popped)
    void
    on_end_element(const Glib::ustring& name)
    {
      if ( !context_->empty() )
	{
	  context_->top()->intf_->on_end_element(name);

	  delete context_->top();
	}
    }

    // ALingA serialization guarantees that PCDATA does not cross packet boundaries
    void on_characters(const Glib::ustring& characters)
    {
      if ( !context_->empty() )
	context_->top()->intf_->on_characters(characters);
    }

    void on_cdata_block(const Glib::ustring& text)
    {
      if ( !context_->empty() )
	context_->top()->intf_->on_cdata_block(text);
    }
    
  };

}
#endif
